library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;


entity vga_text_core is
    Port ( clkvga  : in	 	std_logic;
           res 		 : in	 	std_logic;

					 pixoff  : out  std_logic;	-- pixel aus (alles auf 0)

           addr    : out  std_logic_vector (13 downto 0);  -- addresse auf zeichen im text-ram

           h       : out std_logic; -- horizontal sync
           v       : out std_logic; -- vertikal sync

           h_pos   : out std_logic_vector ( 9 downto 0); -- akt. spalte
           v_pos   : out std_logic_vector ( 9 downto 0)  -- akt. zeile
      	 );
      	 
end vga_text_core;

architecture Behavioral of vga_text_core is

constant h_count_max   : std_logic_vector ( 9 downto 0) := "1010000000"; -- 640
constant h_front_porch : std_logic_vector ( 9 downto 0) := "0000010000"; -- 16
constant h_sync_pulse  : std_logic_vector ( 9 downto 0) := "0001100000"; -- 96
constant h_back_porch  : std_logic_vector ( 9 downto 0) := "0000110000"; -- 48

constant v_count_max   : std_logic_vector ( 9 downto 0) := "0111100000"; -- 480
constant v_front_porch : std_logic_vector ( 9 downto 0) := "0000001011"; -- 11
constant v_sync_pulse  : std_logic_vector ( 9 downto 0) := "0000000010"; -- 2
constant v_back_porch  : std_logic_vector ( 9 downto 0) := "0000011111"; -- 31

constant a_line_inc		 : std_logic_vector ( 6 downto 0) := "1010000";


signal v_count 	: std_logic_vector ( 9 downto 0);
signal h_count 	: std_logic_vector ( 9 downto 0);
signal t_addr  	: std_logic_vector (13 downto 0);

signal h_state 	: std_logic_vector ( 1 downto 0);
signal v_state 	: std_logic_vector ( 1 downto 0);

signal h_out : std_logic;
signal v_out : std_logic;

--signal v_pos_z1 	: std_logic_vector ( 9 downto 0);
--signal h_pos_z1 	: std_logic_vector ( 9 downto 0);


begin
  process (clkvga, res)
  begin
    if res = '1' then
      h_state 	<= (others => '0');
      v_state 	<= (others => '0');

      h_pos   	<= (others => '0');
      v_pos	   <= (others => '0');
      
      v_count 	<= (others => '0');
      h_count 	<= (others => '0');
		t_addr		 <= (others => '0');
		
     	h      <= '0';
		v      <= '0';
     	h_out  <= '0';
		v_out  <= '0';
		
    elsif clkvga'event and clkvga = '1' then
     
      h_count <= h_count + '1';  -- horizontal zhler
     	h       <= h_out;           -- syncs versetzt ausgeben
		v       <= v_out;			
		addr    <= t_addr;
		
		--h_pos_z1 <= h_count;
      --v_pos_z1 <= v_count;
      
      h_pos <= h_count;
      v_pos <= v_count;		
      
      if (h_state = "00") and (v_state = "00") then  
         -- adressen und zhler nur bei bildausgabe ausgeben (sonst so stehen lassen)

			t_addr <= h_count (9 downto 3) + a_line_inc * v_count (9 downto 3);

 			pixoff 	<= '0';
         v_out 		<= '1';
         h_out 		<= '1';                                
      else
			pixoff 	<= '1';
			  
			h_pos <= (others => '0');
	      v_pos <= (others => '0');
      
        if v_state = "00" then
          case h_state is  
            when "01" => h_out <= '1';
                         v_out <= '1';
            when "10" => h_out <= '0';
                         v_out <= '1';
            when "11" => h_out <= '1';
                         v_out <= '1';
            when others => null;                         
         end case;   
            
        elsif v_state = "01" then
          case h_state is  
            when "01" => h_out <= '1';
                         v_out <= '1';
            when "10" => h_out <= '0';
                         v_out <= '1';
            when "11" => h_out <= '1';
                         v_out <= '1';
            when others => null;                         
         end case;  
         
        elsif v_state = "10" then
          case h_state is  
            when "01" => h_out <= '1';
                         v_out <= '0';
            when "10" => h_out <= '0';
                         v_out <= '0';
            when "11" => h_out <= '1';
                         v_out <= '0';
            when others => null;                         
         end case;          
         
        elsif v_state = "11" then        
          case h_state is  
            when "01" => h_out <= '1';
                         v_out <= '1';
            when "10" => h_out <= '0';
                         v_out <= '1';
            when "11" => h_out <= '1';
                         v_out <= '1';
                         t_addr <= (others => '0'); -- zhler zurcksetzen
            when others => null;                         
         end case;                  
        end if;
      end if;
      
      case h_state is
        when "00"   => if h_count = h_count_max - 1 then
                         h_state <= "01";
                         h_count <= (others => '0');
                       end if;
        when "01"   => if h_count = h_back_porch - 1 then
                         h_state <= "10";
                         h_count <= (others => '0');
                       end if;
        when "10"   => if h_count = h_sync_pulse - 1 then
                         h_state <= "11";
                         h_count <= (others => '0');
                       end if;
        when "11"   => if h_count = h_front_porch - 1 then
                         h_state <= "00";
                         h_count <= (others => '0');
                         v_count <= v_count + '1';
                       end if;                                                                     

        when others => null;  
      end case;  
      
      case v_state is
        when "00" => if v_count = v_count_max then
            
                        v_count <= (others => '0');
                        v_state <= "01";
                     end if;
        when "01" => if v_count = v_back_porch then
            
                        v_count <= (others => '0');
                        v_state <= "10";
                     end if;
        when "10" => if v_count = v_sync_pulse then
            
                        v_count <= (others => '0');
                        v_state <= "11";
                     end if;
        when "11" => if v_count = v_front_porch then
            
                        v_count <= (others => '0');
                        v_state <= "00";
                     end if; 
        when others => null;                                                              
      end case;
      
    end if;
  end process;  
  
end Behavioral;
