library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

--library work;
--use work.vga_core.all;

entity vga_test1 is
end vga_test1;

architecture Behavioral of vga_test1 is

component vga_text_core
    Port ( clkvga  : in	 	std_logic;
           res 		 : in	 	std_logic;

					 pixoff  : out  std_logic;	-- pixel aus (alles auf 0)

           addr    : out  std_logic_vector (13 downto 0);  -- addresse auf zeichen im text-ram
					            
           h       : out std_logic; -- horizontal sync
           v       : out std_logic; -- vertikal sync

           h_pos   : out std_logic_vector ( 9 downto 0); -- akt. spalte
           v_pos   : out std_logic_vector ( 9 downto 0)  -- akt. zeile
      	 );
end component;

component vga_text_ram
    Port ( clkvga  : in	 	std_logic;
           clkti   :  in std_logic;

			  	 gcaddr  :  in  std_logic_vector (13 downto 0);  -- lese-addresse auf zeichen im text-ram           
					 gcdata  :  out std_logic_vector ( 7 downto 0);  -- lese-addresse auf zeichen im text-ram   

           tiaddr   :  in  std_logic_vector (13 downto 0);  -- lese-addresse auf zeichen im text-ram
           tiwdata  :  in  std_logic_vector ( 7 downto 0);
           tirdata  :  out std_logic_vector ( 7 downto 0);

					 tiwen    :  in std_logic	
      	 );
 end component;      	 
      	 

   signal clkvga  : std_logic;
   signal res 		  : std_logic;

   signal pixoff  : std_logic;	-- pixel aus (alles auf 0)

   signal addr    : std_logic_vector (13 downto 0);  -- addresse auf zeichen im text-ram
					            
   signal h       : std_logic; -- horizontal sync
   signal v       : std_logic; -- vertikal sync

   signal h_pos   : std_logic_vector ( 9 downto 0); -- akt. spalte
   signal v_pos   : std_logic_vector ( 9 downto 0);  -- akt. zeile

   signal cur_char : std_logic_vector ( 7 downto 0);
   signal cur_char2 : std_logic_vector ( 7 downto 0);
   
   constant MCLK_PER         : time := 40 ns;

begin

	vga_text : 	component vga_text_core port map (
								  clkvga 	=> clkvga,
									res		 	=> res,

                           pixoff   => pixoff,
                           
                           addr => addr,
                           
                           h => h,
                           v => v,
                           
                           h_pos => h_pos,
                           v_pos => v_pos
								);
								
	text_ram : component vga_text_ram port map (
	                  clkvga => clkvga,
	                  clkti => clkvga,
	                  
	                  gcaddr => addr,
	                  gcdata => cur_char,
	                  
	                  tiaddr => addr,
	                  tiwdata => (others => '0'),
	                  tirdata => cur_char2,
	                  tiwen  => '0'
	                  );

	RESET_GEN : process
	begin
     	res <= transport '1';
		wait for 10 * MCLK_PER;
	 	res <= transport '0';
	 	wait;				  
	end process;

  CLOCK_GEN : process
    begin  -- 25 mhz
       CLOCK_LOOP : loop
           clkvga <= transport '1';
           wait for MCLK_PER/2;
           clkvga <= transport '0';
           wait for MCLK_PER/2;
       end loop CLOCK_LOOP;
   	end process;
       


end Behavioral;
